@@ -7,13 +7,14 @@ Points and areas will be processed, unclosed ways will be skipped.
 This example shows how geometries from osmium objects can be imported
 into shapely using the WKBFactory.
 """
-import osmium as o
+import osmium
 import sys
 import shapely.wkb as wkblib
 
-wkbfab = o.geom.WKBFactory()
+wkbfab = osmium.geom.WKBFactory()
 
-class AmenityListHandler(o.SimpleHandler):
+
+class AmenityListHandler(osmium.SimpleHandler):
 
     def print_amenity(self, tags, lon, lat):
         name = tags.get('name', '')
@@ -33,10 +34,11 @@ def main(osmfile):
 
     handler = AmenityListHandler()
 
-    handler.apply_file(osmfile, filters=[o.filter.KeyFilter('amenity')])
+    handler.apply_file(osmfile, filters=[osmium.filter.KeyFilter('amenity')])
 
     return 0
 
+
 if __name__ == '__main__':
     if len(sys.argv) != 2:
         print("Usage: python %s <osmfile>" % sys.argv[0])
