@@ -3,10 +3,11 @@ Simple example that counts the number of objects in an osm file.
 
 Shows how to write a handler for the different types of objects.
 """
-import osmium as o
+import osmium
 import sys
 
-class FileStatsHandler(o.SimpleHandler):
+
+class FileStatsHandler(osmium.SimpleHandler):
 
     def __init__(self):
         super(FileStatsHandler, self).__init__()
@@ -35,6 +36,7 @@ def main(osmfile):
 
     return 0
 
+
 if __name__ == '__main__':
     if len(sys.argv) != 2:
         print("Usage: python %s <osmfile>" % sys.argv[0])
