@@ -2,10 +2,11 @@
  *
  * This file is part of pyosmium. (https://osmcode.org/pyosmium/)
  *
- * Copyright (C) 2023 Sarah Hoffmann <lonvia@denofr.de> and others.
+ * Copyright (C) 2024 Sarah Hoffmann <lonvia@denofr.de> and others.
  * For a full list of authors see the git log.
  */
 #include <pybind11/pybind11.h>
+#include <pybind11/stl/filesystem.h>
 
 #include <osmium/osm.hpp>
 #include <osmium/io/any_output.hpp>
@@ -16,18 +17,33 @@
 
 #include "cast.h"
 #include "osm_base_objects.h"
+#include "base_handler.h"
+
+#include <filesystem>
 
 namespace py = pybind11;
 
 namespace {
 
-class SimpleWriter
+class SimpleWriter : public pyosmium::BaseHandler
 {
     enum { BUFFER_WRAP = 4096 };
 
 public:
-    SimpleWriter(const char* filename, size_t bufsz=4096*1024, osmium::io::Header header=osmium::io::Header())
-    : writer(filename, header),
+    SimpleWriter(const char* filename, size_t bufsz, osmium::io::Header const *header,
+                 bool overwrite, const std::string &filetype)
+    : writer(osmium::io::File(filename, filetype),
+             header ? *header : osmium::io::Header(),
+             overwrite ? osmium::io::overwrite::allow : osmium::io::overwrite::no),
+      buffer(bufsz < 2 * BUFFER_WRAP ? 2 * BUFFER_WRAP : bufsz,
+             osmium::memory::Buffer::auto_grow::yes),
+      buffer_size(buffer.capacity()) // same rounding to BUFFER_WRAP
+    {}
+
+    SimpleWriter(osmium::io::File file, size_t bufsz, osmium::io::Header const *header,
+                 bool overwrite)
+    : writer(file, header ? *header : osmium::io::Header(),
+             overwrite ? osmium::io::overwrite::allow : osmium::io::overwrite::no),
       buffer(bufsz < 2 * BUFFER_WRAP ? 2 * BUFFER_WRAP : bufsz,
              osmium::memory::Buffer::auto_grow::yes),
       buffer_size(buffer.capacity()) // same rounding to BUFFER_WRAP
@@ -36,6 +52,34 @@ public:
     virtual ~SimpleWriter()
     { close(); }
 
+
+    bool node(pyosmium::PyOSMNode &o) override
+    {
+        buffer.add_item(*(o.get()));
+        flush_buffer();
+        return false;
+    }
+
+    bool way(pyosmium::PyOSMWay &o) override
+    {
+        buffer.add_item(*(o.get()));
+        flush_buffer();
+        return false;
+    }
+
+    bool relation(pyosmium::PyOSMRelation &o) override
+    {
+        buffer.add_item(*(o.get()));
+        flush_buffer();
+        return false;
+    }
+
+    void flush() override
+    {
+        flush_buffer(true);
+    }
+
+
     void add_node(py::object o)
     {
         if (!buffer) {
@@ -44,7 +88,7 @@ public:
 
         buffer.rollback();
 
-        auto const *inode = pyosmium::try_cast<COSMNode>(o);
+        auto const *inode = pyosmium::try_cast<pyosmium::COSMNode>(o);
         if (inode) {
             buffer.add_item(*inode->get());
         } else {
@@ -57,7 +101,7 @@ public:
             }
 
             set_common_attributes(o, builder);
-            set_taglist<COSMNode>(o, builder);
+            set_taglist<pyosmium::COSMNode>(o, builder);
         }
 
         flush_buffer();
@@ -71,7 +115,7 @@ public:
 
         buffer.rollback();
 
-        auto const *iobj = pyosmium::try_cast<COSMWay>(o);
+        auto const *iobj = pyosmium::try_cast<pyosmium::COSMWay>(o);
         if (iobj) {
             buffer.add_item(*iobj->get());
         } else {
@@ -79,7 +123,7 @@ public:
 
             set_common_attributes(o, builder);
             set_nodelist(o, &builder);
-            set_taglist<COSMWay>(o, builder);
+            set_taglist<pyosmium::COSMWay>(o, builder);
         }
 
         flush_buffer();
@@ -93,7 +137,7 @@ public:
 
         buffer.rollback();
 
-        auto const *iobj = pyosmium::try_cast<COSMRelation>(o);
+        auto const *iobj = pyosmium::try_cast<pyosmium::COSMRelation>(o);
         if (iobj) {
             buffer.add_item(*iobj->get());
         } else {
@@ -101,7 +145,7 @@ public:
 
             set_common_attributes(o, builder);
             set_memberlist(o, &builder);
-            set_taglist<COSMRelation>(o, builder);
+            set_taglist<pyosmium::COSMRelation>(o, builder);
         }
 
         flush_buffer();
@@ -242,7 +286,7 @@ private:
         }
 
         // original memberlist
-        auto const &iobj = pyosmium::try_cast<COSMRelation>(o);
+        auto const &iobj = pyosmium::try_cast<pyosmium::COSMRelation>(o);
         if (iobj) {
             auto const &oml = iobj->get()->members();
             if (oml.size() > 0)
@@ -283,11 +327,11 @@ private:
         return osmium::Location{l[0].cast<double>(), l[1].cast<double>()};
     }
 
-    void flush_buffer()
+    void flush_buffer(bool force = false)
     {
         buffer.commit();
 
-        if (buffer.committed() > buffer_size - BUFFER_WRAP) {
+        if (force || buffer.committed() > buffer_size - BUFFER_WRAP) {
             osmium::memory::Buffer new_buffer(buffer_size, osmium::memory::Buffer::auto_grow::yes);
             using std::swap;
             swap(buffer, new_buffer);
@@ -300,38 +344,47 @@ private:
     size_t buffer_size;
 };
 
-}
+} // namespace
+
+namespace pyosmium {
 
 void init_simple_writer(pybind11::module &m)
 {
-    py::class_<SimpleWriter>(m, "SimpleWriter",
-        "The most generic class to write osmium objects into a file. The writer "
-        "takes a file name as its mandatory parameter. The file must not yet "
-        "exist. The file type to output is determined from the file extension. "
-        "The second (optional) parameter is the buffer size. osmium caches the "
-        "output data in an internal memory buffer before writing it on disk. This "
-        "parameter allows changing the default buffer size of 4MB. Larger buffers "
-        "are normally better but you should be aware that there are normally multiple "
-        "buffers in use during the write process.")
-        .def(py::init<const char*, unsigned long, osmium::io::Header>())
-        .def(py::init<const char*, unsigned long>())
-        .def(py::init<const char*>())
-        .def("add_node", &SimpleWriter::add_node, py::arg("node"),
-             "Add a new node to the file. The node may be an ``osmium.osm.Node`` object, "
-             "an ``osmium.osm.mutable.Node`` object or any other Python object that "
-             "implements the same attributes.")
-        .def("add_way", &SimpleWriter::add_way, py::arg("way"),
-             "Add a new way to the file. The way may be an ``osmium.osm.Way`` object, "
-             "an ``osmium.osm.mutable.Way`` object or any other Python object that "
-             "implements the same attributes.")
-        .def("add_relation", &SimpleWriter::add_relation, py::arg("relation"),
-             "Add a new relation to the file. The relation may be an "
-             "``osmium.osm.Relation`` object, an ``osmium.osm.mutable.Relation`` "
-             "object or any other Python object that implements the same attributes.")
-        .def("close", &SimpleWriter::close,
-             "Flush the remaining buffers and close the writer. While it is not "
-             "strictly necessary to call this function explicitly, it is still "
-             "strongly recommended to close the writer as soon as possible, so "
-             "that the buffer memory can be freed.")
+    py::class_<SimpleWriter, BaseHandler>(m, "SimpleWriter")
+        .def(py::init<const char*, unsigned long, osmium::io::Header const *, bool, const std::string&>(),
+             py::arg("filename"), py::arg("bufsz") = 4096*1024,
+             py::arg("header") = nullptr,
+             py::arg("overwrite") = false,
+             py::arg("filetype") = "")
+        .def(py::init<>([] (std::filesystem::path const &file, unsigned long bufsz,
+                            osmium::io::Header const *header, bool overwrite) {
+                 return new SimpleWriter(file.string().c_str(), bufsz, header, overwrite, "");
+             }),
+             py::arg("filename"), py::arg("bufsz") = 4096*1024,
+             py::arg("header") = nullptr,
+             py::arg("overwrite") = false)
+        .def(py::init<osmium::io::File, unsigned long, osmium::io::Header const *, bool>(),
+             py::arg("filename"), py::arg("bufsz") = 4096*1024,
+             py::arg("header") = nullptr,
+             py::arg("overwrite") = false)
+        .def("add_node", &SimpleWriter::add_node, py::arg("node"))
+        .def("add_way", &SimpleWriter::add_way, py::arg("way"))
+        .def("add_relation", &SimpleWriter::add_relation, py::arg("relation"))
+        .def("add", [](SimpleWriter &self, py::object const &o) {
+                           if (py::isinstance<pyosmium::COSMNode>(o) || py::hasattr(o, "location")) {
+                               self.add_node(o);
+                           } else if (py::isinstance<pyosmium::COSMWay>(o) || py::hasattr(o, "nodes")) {
+                               self.add_way(o);
+                           } else if (py::isinstance<pyosmium::COSMRelation>(o) || py::hasattr(o, "members")) {
+                               self.add_relation(o);
+                           } else {
+                               throw py::type_error("Need node, way or relation object.");
+                           }
+                    })
+        .def("close", &SimpleWriter::close)
+        .def("__enter__", [](py::object const &self) { return self; })
+        .def("__exit__", [](SimpleWriter &self, py::args args) { self.close(); })
     ;
 }
+
+} // namespace
