@@ -10,10 +10,11 @@ from textwrap import dedent
 import uuid
 
 import pytest
+import requests
 
 from werkzeug.wrappers import Response
 
-from helpers import mkdate, CountingHandler
+from helpers import mkdate, CountingHandler, IDCollector
 
 import osmium.replication.server as rserv
 import osmium.replication
@@ -223,6 +224,33 @@ def test_apply_diffs_count(httpserver):
         assert h.counts == [1, 1, 1, 0]
 
 
+@pytest.mark.parametrize('end_id,max_size, actual_end', [(107, None, 107),
+                                                         (None, 512, 108),
+                                                         (105, 512, 105),
+                                                         (110, 512, 108),
+                                                         (None, None, 115)])
+def test_apply_diffs_endid(httpserver, end_id, max_size, actual_end):
+    httpserver.expect_request('/state.txt').respond_with_data("""\
+        sequenceNumber=140
+        timestamp=2017-08-26T11\\:04\\:02Z
+    """)
+    for i in range(100, 141):
+        httpserver.expect_request(f'/000/000/{i}.opl')\
+                  .respond_with_data(f"r{i} M" + ",".join(f"n{i}@" for i in range(1, 3000)))
+
+    with rserv.ReplicationServer(httpserver.url_for(''), "opl") as svr:
+        res = svr.collect_diffs(101, end_id=end_id, max_size=max_size)
+
+        assert res is not None
+        assert res.id == actual_end
+        assert res.newest == 140
+
+        ids = IDCollector()
+        res.reader.apply(ids)
+
+        assert ids.relations == list(range(101, actual_end + 1))
+
+
 def test_apply_diffs_without_simplify(httpserver):
     httpserver.expect_ordered_request('/state.txt').respond_with_data("""\
         sequenceNumber=100
@@ -364,6 +392,23 @@ def test_apply_diffs_permanent_error(httpserver, caplog):
     httpserver.expect_ordered_request('/000/000/100.opl')\
               .respond_with_data('not a file', status=404)
 
+    with caplog.at_level(logging.ERROR):
+        with rserv.ReplicationServer(httpserver.url_for(''), "opl") as svr:
+            h = CountingHandler()
+            with pytest.raises(requests.HTTPError, match='404'):
+                svr.apply_diffs(h, 100, 10000)
+
+    assert 'Permanent server error' in caplog.text
+
+
+def test_apply_diffs_transient_error_first_diff(httpserver, caplog):
+    httpserver.expect_ordered_request('/state.txt').respond_with_data("""\
+        sequenceNumber=100
+        timestamp=2017-08-26T11\\:04\\:02Z
+    """)
+    httpserver.expect_request('/000/000/100.opl')\
+              .respond_with_data('not a file', status=503)
+
     with caplog.at_level(logging.ERROR):
         with rserv.ReplicationServer(httpserver.url_for(''), "opl") as svr:
             h = CountingHandler()
@@ -394,7 +439,7 @@ def test_apply_diffs_permanent_error_later_diff(httpserver, caplog):
     assert 'Error during diff download' in caplog.text
 
 
-def test_apply_diffs_transient_error(httpserver, caplog):
+def test_apply_diffs_transient_error_later_diff(httpserver, caplog):
     httpserver.expect_ordered_request('/state.txt').respond_with_data("""\
         sequenceNumber=101
         timestamp=2017-08-26T11\\:04\\:02Z
